@@ -1293,6 +1293,16 @@ _cairo_mime_data_destroy (void *ptr)
  * discarded if you draw on the surface afterwards. Use this function
  * with care.
  *
+ * Even if a backend supports a MIME type, that does not mean cairo
+ * will always be able to use the attached MIME data. For example, if
+ * the backend does not natively support the compositing operation used
+ * to apply the MIME data to the backend. In that case, the MIME data
+ * will be ignored. Therefore, to apply an image in all cases, it is best
+ * to create an image surface which contains the decoded image data and
+ * then attach the MIME data to that. This ensures the image will always
+ * be used while still allowing the MIME data to be used whenever
+ * possible.
+ *
  * Return value: %CAIRO_STATUS_SUCCESS or %CAIRO_STATUS_NO_MEMORY if a
  * slot could not be allocated for the user data.
  *
@@ -1650,10 +1660,10 @@ slim_hidden_def (cairo_surface_mark_dirty_rectangle);
 /**
  * cairo_surface_set_device_scale:
  * @surface: a #cairo_surface_t
- * @sx: a scale factor in the X direction
- * @sy: a scale factor in the Y direction
+ * @x_scale: a scale factor in the X direction
+ * @y_scale: a scale factor in the Y direction
  *
- * Sets an scale that is multiplied to the device coordinates determined
+ * Sets a scale that is multiplied to the device coordinates determined
  * by the CTM when drawing to @surface. One common use for this is to
  * render to very high resolution display devices at a scale factor, so
  * that code that assumes 1 pixel will be a certain size will still work.
@@ -1668,8 +1678,8 @@ slim_hidden_def (cairo_surface_mark_dirty_rectangle);
  **/
 void
 cairo_surface_set_device_scale (cairo_surface_t *surface,
-				double		 sx,
-				double		 sy)
+				double		 x_scale,
+				double		 y_scale)
 {
     cairo_status_t status;
 
@@ -1689,8 +1699,8 @@ cairo_surface_set_device_scale (cairo_surface_t *surface,
 	return;
     }
 
-    surface->device_transform.xx = sx;
-    surface->device_transform.yy = sy;
+    surface->device_transform.xx = x_scale;
+    surface->device_transform.yy = y_scale;
     surface->device_transform.xy = 0.0;
     surface->device_transform.yx = 0.0;
 
@@ -2615,6 +2625,16 @@ _cairo_surface_set_resolution (cairo_surface_t *surface,
     surface->y_resolution = y_res;
 }
 
+/**
+ * _cairo_surface_create_in_error:
+ * @status: the error status
+ *
+ * Return an appropriate static error surface for the error status.
+ * On error, surface creation functions should always return a surface
+ * created with _cairo_surface_create_in_error() instead of a new surface
+ * in an error state. This simplifies internal code as no refcounting has
+ * to be done.
+ **/
 cairo_surface_t *
 _cairo_surface_create_in_error (cairo_status_t status)
 {
